import { HttpService } from '@nestjs/axios';
import { Injectable, UnauthorizedException } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { RESTResponseBody, RESTError } from '../models/dtos/wrappers/rest-wrapper.class';
import { ISession } from '../models/interfaces/session.interface';
import { firstValueFrom } from 'rxjs';
import { Model } from 'mongoose';
import { Session, SessionDocument } from '../models/schemas/session.schema';
import { InjectModel } from '@nestjs/mongoose';

@Injectable()
export class SessionsService {
    //#region Private Members
    private _baseUrl: string;
    //#endregion

    //#region Constructor
    constructor(
        @InjectModel(Session.name, 'sessions-store')
        private readonly _sessionModel: Model<SessionDocument>,
        private readonly _httpService: HttpService,
        private readonly _configService: ConfigService,
    ) {
        this._baseUrl = this._configService.getOrThrow('DL_SESSIONS__REST__BASE_URL');
    }
    //#endregion

    //#region Public API
    public async create(userId: string, userAgent: string, ip: string) {
        try {
            const endpoint = `${this._baseUrl}/v1/sessions/${userId}`;
            const headers = { 'user-agent': userAgent, 'x-ip': ip };
            const response$ = this._httpService.post<RESTResponseBody<ISession>>(endpoint, null, { headers });
            const response = await firstValueFrom(response$);

            if (response.data.errors) {
                throw new RESTError('sessions_api_error', response.data.errors);
            }
            if (!response.data || !response.data.result) {
                throw new RESTError('sessions_api_error', 'empty response');
            }
            return response.data.result.data;
        } catch (e) {
            throw RESTError.createFromCatch(e, { keyword: 'unknown', debug: e });
        }
    }

    public async read(token: string): Promise<ISession> {
        if (!token) {
            throw new UnauthorizedException();
        }

        const session = await this._sessionModel.findOne({ token });
        if (!session) {
            throw new UnauthorizedException();
        }

        return session;
    }
    //#endregion
}
